/*
 * HNReader for Android, with preloading.
 * You can preload the content of HN Links and read the articles + comments when you're offline.
 * @author Vadim Kr.
 * @copyright (c) 2013 bndr
 * @license http://creativecommons.org/licenses/by-sa/3.0/legalcode
 */
package com.example.hnreader;


import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.text.Html;
import android.view.Menu;
import android.widget.TextView;
import com.example.hnreader.utility.Database;
import java.util.ArrayList;


public class SingleView extends Activity {

    private NewsItem ni;
    private Handler mHandler;
    public ProgressDialog dialog;
    private Database db;
    private TextView article_text;
    private TextView article_title;
    private TextView article_comments;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.single_view);

        Intent intent = getIntent();
        ni = (NewsItem) intent.getSerializableExtra("item");
        db = Database.getInstance(getApplicationContext());
        mHandler = new Handler();

        article_text = (TextView) findViewById(R.id.article_text);
        article_title = (TextView) findViewById(R.id.article_title);
        article_comments = (TextView) findViewById(R.id.article_comments);
        article_text.setLineSpacing(4, (float) 1.6);
        article_comments.setLineSpacing(3, (float) 1.3);

        NewsItem full = db.getItem(ni.url);
        article_title.setText(ni.title);

        if (full != null) {

            article_text.setText(Html.fromHtml(full.text));
            if (full.comments != null) {
                article_comments.setText(Html.fromHtml(full.comments));
            }

        } else {
            showLoading(false);
            Thread tr = new Thread(
                    new Runnable() {
                        public void run() {
                            ni.text = db.getArticle(ni.url);
                            ni.comments = db.getStringFromArray(db.getComments(ni.comments_link));
                            mHandler.post(new Runnable() {
                                public void run() {
                                    article_text.setText(Html.fromHtml(ni.text));
                                    article_comments.setText(Html.fromHtml(ni.comments));
                                    ArrayList<NewsItem> items = new ArrayList<NewsItem>();
                                    items.add(ni);
                                    db.insertNewItems(items);
                                    dialog.dismiss();
                                }
                            });
                        }
                    });
            tr.start();
        }

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.hnreader, menu);
        return true;
    }

    private void showLoading(boolean dismiss) {
        if (dismiss) {
            dialog.hide();
            return;
        }
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setMessage("Loading..Please wait");
            dialog.setProgress(0);
            dialog.setCancelable(false);
        }
        dialog.show();


    }

}
