/*
 * HNReader for Android, with preloading.
 * You can preload the content of HN Links and read the articles + comments when you're offline.
 * @author Vadim Kr.
 * @copyright (c) 2013 bndr
 * @license http://creativecommons.org/licenses/by-sa/3.0/legalcode
 */
package com.example.hnreader.utility;


import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;
import com.example.hnreader.Download;
import com.example.hnreader.NewsItem;
import com.example.hnreader.models.Comment;
import com.example.hnreader.models.EntriesContract;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.util.ArrayList;
import java.util.concurrent.ExecutionException;


/**
 * Created by bndr on 5/19/13.
 */
public class Database extends SQLiteOpenHelper {

    // If you change the database schema, you must increment the database version.
    public static final int DATABASE_VERSION = 5;
    public static final String DATABASE_NAME = "HNitems.db";
    private static Database dinstance = null;
    private SQLiteDatabase db;
    private ContentValues values;
    private Context context;

    /**
     * Only one instance of database should be created.
     *
     * @param ctx
     * @return
     */
    public static Database getInstance(Context ctx) {

        if (dinstance == null) {
            dinstance = new Database(ctx);
        }
        return dinstance;
    }

    /**
     * Constructor
     *
     * @param context
     */
    public Database(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
        this.context = context;

        db = this.getWritableDatabase();
        values = new ContentValues();
    }


    public void onCreate(SQLiteDatabase db) {
        SQLiteDatabase checkDB = null;
        Log.d("DEBUG", "ONCREATE CALLED");

    }

    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        Log.d("DEBUG", "ON UPGRADE CALLED");
        // This database is only a cache for online data, so its upgrade policy is
        // to simply to discard the data and start over
        //  db.execSQL(EntriesContract.SQL_DELETE_ENTRIES);
        //  onCreate(db);
    }

    public void onDowngrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        onUpgrade(db, oldVersion, newVersion);
    }

    /**
     * Insert new items to the database.
     *
     * @param items_list
     */
    public void insertNewItems(ArrayList<NewsItem> items_list) {

        db.beginTransaction();
        for (NewsItem item : items_list) {

            values.put(EntriesContract.Entry.COLUMN_NAME_ENTRY_ID, item.id);
            values.put(EntriesContract.Entry.COLUMN_NAME_TITLE, item.title);
            values.put(EntriesContract.Entry.COLUMN_NAME_URL, item.url);
            values.put(EntriesContract.Entry.COLUMN_NAME_AUTHOR, item.username);
            values.put(EntriesContract.Entry.COLUMN_NAME_DOMAIN, item.domain);
            values.put(EntriesContract.Entry.COLUMN_NAME_TEXT, item.text);
            values.put(EntriesContract.Entry.COLUMN_NAME_DATE, item.date);
            values.put(EntriesContract.Entry.COLUMN_NAME_POINTS, item.points);
            values.put(EntriesContract.Entry.COLUMN_NAME_COMMENTS_COUNT, item.comments_count);
            values.put(EntriesContract.Entry.COLUMN_NAME_COMMENTS, item.comments);
            long newRowId;
            newRowId = db.insert(EntriesContract.Entry.TABLE_NAME, null, values);
        }
        db.setTransactionSuccessful();
        db.endTransaction();

    }

    /**
     * Get Item from the database by url.
     *
     * @param id
     * @return
     */
    public NewsItem getItem(String id) {
        Cursor c = null;
        NewsItem ni = null;
        c = db.query(EntriesContract.Entry.TABLE_NAME, null, EntriesContract.Entry.COLUMN_NAME_URL + " = ?", new String[]{id}, null, null, null);
        c.moveToFirst();

        int id_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_ENTRY_ID);
        int url_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_URL);
        int title_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_TITLE);
        int author_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_AUTHOR);
        int cc_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_COMMENTS_COUNT);
        int date_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_DATE);
        int domain_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_DOMAIN);
        int text_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_TEXT);
        int comments_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_COMMENTS);
        int points_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_POINTS);

        if (c.getCount() < 1) {
            Log.d("DEBUG", "GET ITEM IS EMPTY. SIZE LESS THAN ONE");
            return null;
        }
        c.moveToFirst();
        ni = new NewsItem(c.getString(title_i), c.getString(id_i), c.getString(url_i), Integer.parseInt(c.getString(points_i)), c.getString(date_i), Integer.parseInt(c.getString(cc_i)), c.getString(author_i), "");
        ni.text = c.getString(text_i);
        ni.comments = c.getString(comments_i);
        ni.domain = c.getString(domain_i);

        return ni;
    }

    /**
     * Get All items in the database.
     *
     * @return
     */
    public ArrayList<NewsItem> getAll() {
        ArrayList<NewsItem> an = new ArrayList<NewsItem>();
        Cursor c = null;
        NewsItem ni = null;

        db.beginTransaction();
        c = db.rawQuery("SELECT * FROM " + EntriesContract.Entry.TABLE_NAME, null);
        int id_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_ENTRY_ID);
        int url_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_URL);
        int title_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_TITLE);
        int author_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_AUTHOR);
        int cc_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_COMMENTS_COUNT);
        int date_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_DATE);
        int domain_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_DOMAIN);
        int points_i = c.getColumnIndex(EntriesContract.Entry.COLUMN_NAME_POINTS);
        c.moveToFirst();
        if (c.getCount() < 1) {
            return an;
        }

        do {
            ni = new NewsItem(c.getString(title_i), c.getString(id_i), c.getString(url_i), Integer.parseInt(c.getString(points_i)), c.getString(date_i), Integer.parseInt(c.getString(cc_i)), c.getString(author_i), "");
            an.add(ni);
        } while (c.moveToNext());

        db.setTransactionSuccessful();
        db.endTransaction();
        return an;
    }

    /**
     * Get Article Data from webpage. If no text with the most element is found
     * Try to find element with predefined id or classname
     *
     * @param url
     * @return
     */
    public String getArticle(String url) {
        String art = "";
        String text = "";
        String[] variants = {"entry", "entry-content", "article-content", "post", "item text", "article", "txt_content", "post-body", "post-field", "content", "instapaper_body", "storytext", "body-copy", "news_body", "main"};
        String[] variants_id = {"singleBlogPost", "article"};
        try {
            art = new Download(context).execute(url).get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }

        Document doc = Jsoup.parse(art);
        Element article = checkP(doc);

        if (article != null) {
            return article.html();
        }
        for (int i = 0; i < variants.length; i++) {
            Elements entry = doc.getElementsByClass(variants[i]);
            if (entry != null && entry.size() >= 1 && entry.first().html().length() > 400) {
                return entry.first().html();
            }
        }
        for (int i = 0; i < variants_id.length; i++) {
            Element entry = doc.getElementById(variants_id[i]);
            if (entry != null && entry.hasText() && entry.html().length() > 400) {
                return entry.html();
            }
        }

        return "";

    }

    /**
     * Find the Element with the most text in it.
     *
     * @param doc
     * @return
     */
    public Element checkP(Document doc) {
        doc.select("img").remove();

        ArrayList<Element> parents = new ArrayList<Element>();
        Element highest_element = null;
        int highest_count = 0;

        Elements ps = doc.getElementsByTag("p");

        for (Element p : ps) {

            if (p == null || p.text().isEmpty()) {
                continue;
            }

            if (p.text().length() > 140) {
                parents.add(p.parent());
            }
        }
        int i = 0;
        for (Element str : parents) {

            for (Element s : parents) {
                if (str.equals(s)) {
                    if (i >= highest_count) {
                        highest_element = str;
                        highest_count = i;
                    }
                    i++;
                }
            }
            i = 0;

        }
        if (highest_element != null) {
            Log.d("DEBUG", "Highest element is CLASS: " + highest_element.className() + ", ID: " + highest_element.id() + " URL IS: " + doc.getElementsByTag("title").first().text());
        }
        return highest_element;


    }

    /**
     * Get Comments from Hackernews webpage
     *
     * @param url
     * @return
     */
    public ArrayList<Comment> getComments(String url) {
        ArrayList<Comment> list = new ArrayList<Comment>();
        String art = "";
        try {
            art = new Download(context).execute(url).get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }

        Document doc = Jsoup.parse(art, "UTF-8");
        Elements comments = doc.select("td.default");

        for (int i = 0; i < comments.size(); i++) {
            Element cm = comments.get(i);
            String ago = cm.select("span.comhead").text();
            if (ago.isEmpty()) {
                continue;
            }
            String username = cm.select("a").first().text();
            String text = cm.select("font").first().text();

            Comment comment = new Comment(username, text, ago);
            list.add(comment);

        }
        return list;

    }

    /**
     * Format the comments
     *
     * @param cm
     * @return
     */
    public String getStringFromArray(ArrayList<Comment> cm) {
        StringBuilder sb = new StringBuilder();

        for (int i = 0; i < cm.size(); i++) {
            sb.append("<p>");
            sb.append("<span>").append(cm.get(i).username).append("</span>");
            sb.append("<span>").append(cm.get(i).ago).append("</span>");
            sb.append("<blockquote>").append(cm.get(i).text).append("</blockquote>");
            sb.append("</p>");

        }
        return sb.toString();
    }

    /**
     * Refresh the database. Clear the cache.
     *
     * @return
     */
    public boolean refresh() {
        db.execSQL(EntriesContract.SQL_DELETE_ENTRIES);
        db.execSQL(EntriesContract.SQL_CREATE_ENTRIES);
        return true;
    }

    /**
     * Get Current database size;
     *
     * @return
     */
    public long getCurrentSize() {
        Cursor c;
        c = db.rawQuery("pragma page_size", null);
        c.moveToFirst();
        int page_size = c.getInt(0);

        c = db.rawQuery("pragma page_count", null);
        c.moveToFirst();
        int page_count = c.getInt(0);
        return page_size * page_count;

    }

}
