/*
 * HNReader for Android, with preloading.
 * You can preload the content of HN Links and read the articles + comments when you're offline.
 * @author Vadim Kr.
 * @copyright (c) 2013 bndr
 * @license http://creativecommons.org/licenses/by-sa/3.0/legalcode
 */

package com.example.hnreader.utility;

import android.util.Log;

import com.example.hnreader.NewsItem;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

public class XMLParser {


    public XMLParser() {

    }

    /**
     * Parse XML as DOM object
     *
     * @param xml
     * @return
     */
    public Document getDomElement(String xml) {
        Document doc = null;
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {

            DocumentBuilder db = dbf.newDocumentBuilder();

            InputSource is = new InputSource();
            is.setCharacterStream(new StringReader(xml));
            doc = db.parse(is);

        } catch (ParserConfigurationException e) {
            Log.e("Error: ", e.getMessage());
            return null;
        } catch (SAXException e) {
            Log.e("Error: ", e.getMessage());
            return null;
        } catch (IOException e) {
            Log.e("Error: ", e.getMessage());
            return null;
        }
        // return DOM
        return doc;
    }

    /**
     * Get Element node value
     *
     * @param item
     * @param str
     * @return
     */
    public String getValue(Element item, String str) {
        NodeList n = item.getElementsByTagName(str);
        return this.getElementValue(n.item(0));
    }

    /**
     * Get the inside data of tag.
     *
     * @param elem
     * @return
     */
    public final String getElementValue(Node elem) {
        Node child;
        if (elem != null) {
            if (elem.hasChildNodes()) {
                for (child = elem.getFirstChild(); child != null; child = child.getNextSibling()) {
                    if (child.getNodeType() == Node.TEXT_NODE) {
                        return child.getNodeValue();
                    }
                }
            }
        }
        return "";
    }

    /**
     * Get ArrayList of items from XML
     * Items limit is 60
     *
     * @param s
     * @return
     */
    public ArrayList GetRssArray(String s) {
        ArrayList<NewsItem> items = new ArrayList<NewsItem>();
        try {
            Document doc = getDomElement(s);

            NodeList nl = doc.getElementsByTagName("item");
            int limit = 60;
            for (int i = 0; i < limit; i++) {
                Element el = (Element) nl.item(i);
                String title = getValue(el, "title");
                String id = getValue(el, "hnsearch_id");
                String link = getValue(el, "link");
                String date = getValue(el, "date");
                String username = getValue(el, "username");
                String cm_link = getValue(el, "comments");
                int comments_count;
                int points;
                try {
                    comments_count = Integer.parseInt(getValue(el, "num_comments"));
                    points = Integer.parseInt(getValue(el, "points"));
                } catch (NumberFormatException e) {
                    continue;
                }
                NewsItem ni = new NewsItem(title, id, link, points, date, comments_count, username, cm_link);
                items.add(ni);

            }
            return items;

        } catch (Exception e) {
            return items;
        }
    }
}
